Package: aplotExtra
Title: Creating Composite Plots using 'aplot'
Version: 0.0.4
Authors@R: c(
    person(given = "Guangchuang", family = "Yu", email = "guangchuangyu@gmail.com",   role  = c("aut", "cre"), comment = c(ORCID = "0000-0002-6485-8781")),
    person("Shuangbin", "Xu", email = "xshuangbin@163.com", role = "ctb", comment = c(ORCID="0000-0003-3513-5362")),
    person("Chun-Hui","Gao", email="gaospecial@gmail.com", role= "ctb", comment=c(ORCID = "0000-0002-1445-7939")),
    person("Shensuo","Li", email="lishensuo@163.com", role= "ctb", comment=c(ORCID = "0009-0006-3229-3700"))
    )
Description: Many complex plots are actually composite plots, such as 'oncoplot', 'funkyheatmap', 'upsetplot', etc. We can produce subplots using 'ggplot2' and combine them to create composite plots using 'aplot'. In this way, it is easy to customize these complex plots, by adding, deleting or modifying subplots in the final plot. This package provides a set of utilities to help users to create subplots and complex plots.
Depends: R (>= 4.1.0)
Imports: aplot (>= 0.2.3), dplyr, forcats, ggfun (>= 0.1.1), ggplot2,
        grid, maftools, purrr, rlang, stats, tibble, tidyr, utils,
        ggstar, yulab.utils (>= 0.0.8)
Suggests: ggtree, data.table, RColorBrewer, R.utils
URL: https://github.com/YuLab-SMU/aplotExtra
License: Artistic-2.0
Encoding: UTF-8
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-06-12 02:36:09 UTC; HUAWEI
Author: Guangchuang Yu [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6485-8781>),
  Shuangbin Xu [ctb] (ORCID: <https://orcid.org/0000-0003-3513-5362>),
  Chun-Hui Gao [ctb] (ORCID: <https://orcid.org/0000-0002-1445-7939>),
  Shensuo Li [ctb] (ORCID: <https://orcid.org/0009-0006-3229-3700>)
Maintainer: Guangchuang Yu <guangchuangyu@gmail.com>
Repository: CRAN
Date/Publication: 2025-06-12 04:20:02 UTC
Built: R 4.6.0; ; 2026-04-25 13:33:37 UTC; unix
