# Generated by roxygen2: do not edit by hand

export(funky_bar)
export(funky_heatmap)
export(funky_point)
export(funky_text)
export(get_all_subsets)
export(oncoplot)
export(upset_plot)
import(maftools)
importFrom(aplot,insert_bottom)
importFrom(aplot,insert_right)
importFrom(aplot,insert_top)
importFrom(aplot,plot_list)
importFrom(aplot,xlab2)
importFrom(aplot,ylab2)
importFrom(dplyr,mutate)
importFrom(forcats,as_factor)
importFrom(ggfun,theme_blinds)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotation_custom)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,xlab)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylab)
importFrom(ggstar,geom_star)
importFrom(grid,gpar)
importFrom(grid,textGrob)
importFrom(grid,unit)
importFrom(purrr,reduce)
importFrom(rlang,.data)
importFrom(stats,setNames)
importFrom(tibble,rownames_to_column)
importFrom(tidyr,pivot_longer)
importFrom(utils,modifyList)
importFrom(yulab.utils,combinations)
importFrom(yulab.utils,get_fun_from_pkg)
